"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Action = void 0;
const assert_1 = __importDefault(require("assert"));
const api_tools_core_1 = require("@fresha/api-tools-core");
const code_morph_ts_1 = require("@fresha/code-morph-ts");
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
const ts_morph_1 = require("ts-morph");
const ActionParam_1 = require("./ActionParam");
/**
 * Generates code for a single NestJS controller action.
 */
class Action {
    context;
    controller;
    pathUrl;
    httpMethod;
    methodName;
    params;
    returnSchema; // OpenAPI schema
    urlSuffix;
    constructor(context, controller, pathUrl, httpMethod, operation) {
        this.context = context;
        this.controller = controller;
        this.pathUrl = pathUrl;
        this.httpMethod = httpMethod;
        this.methodName = operation.operationId ?? pathUrl;
        this.params = Array.from(operation.parameters(), param => new ActionParam_1.ActionParam(this.context, param));
        if (operation.requestBody) {
            this.params.push(new ActionParam_1.ActionParam(this.context, {
                in: 'body',
                name: 'body',
                schema: (0, openapi_codegen_utils_1.getOperationRequestBodySchema)(operation, this.context.useJsonApi),
            }));
        }
        this.returnSchema = (0, openapi_codegen_utils_1.getOperationDefaultResponseSchema)(operation, this.context.useJsonApi);
        this.urlSuffix = null;
    }
    getUrlSuffix() {
        if (this.urlSuffix == null) {
            const parts = this.pathUrl.split('/').map(part => {
                const norm = part.replace(/\{\w+\}/g, m => `:${m.slice(1, -1)}`);
                return norm;
            });
            let norm = parts.join('/');
            if (norm.startsWith('/:')) {
                norm = norm.slice(1);
            }
            this.urlSuffix = norm.slice(this.controller.getUrlPrefix().length);
        }
        return this.urlSuffix;
    }
    generateCode(classDecl) {
        this.context.logger.info(`Generating code for action ${this.methodName}`);
        (0, code_morph_ts_1.addImportDeclarations)(classDecl.getSourceFile(), {
            '@nestjs/common': [(0, api_tools_core_1.titleCase)(this.httpMethod), 'HttpException'],
            '@fresha/json-api': 'JSONAPI',
        });
        const returnTypeName = (0, api_tools_core_1.titleCase)(`${this.methodName}Response`);
        const dto = this.context.addDTO(returnTypeName, this.returnSchema);
        (0, code_morph_ts_1.addImportDeclaration)(classDecl.getSourceFile(), this.controller.relativeModulePath(dto.outputPath), returnTypeName);
        this.generatePublicMethod(classDecl);
        this.generateHandlerMethod(classDecl);
    }
    generatePublicMethod(classDecl) {
        const methodDecoratorName = (0, api_tools_core_1.titleCase)(this.httpMethod);
        const returnTypeName = (0, api_tools_core_1.titleCase)(`${this.methodName}Response`);
        const handleActionName = (0, api_tools_core_1.camelCase)(`handle-${this.methodName}`);
        const responseDtoName = 'Response';
        const actionFunc = classDecl.addMethod({
            name: this.methodName,
            isAsync: true,
            returnType: `Promise<${returnTypeName}>`,
        });
        (0, code_morph_ts_1.addDecorator)(actionFunc, methodDecoratorName, this.getUrlSuffix() || undefined);
        for (const param of this.params) {
            param.generateCode(actionFunc);
        }
        const usedParamNames = this.params.filter(p => p.from !== 'body').map(p => p.name);
        switch (this.httpMethod) {
            case 'get':
                actionFunc.addStatements(`
          try {
            const result = await this.${handleActionName}(${usedParamNames.join(', ')});
            return JSONAPI.stringify(result, ${responseDtoName});
          } catch (err) {
            if (err instanceof JSONAPI.Exception) {
              throw JSONAPI.throwHttpException(err);
            } else if (!(err instanceof HttpException)) {
              throw new HttpException(String(err), 500);
            } else {
              throw err;
            }
          }
        `);
                break;
            case 'post':
            case 'put':
            case 'patch':
                actionFunc.addStatements(`
          try {
            const params = JSONAPI.parse(body);
            const result = await this.${handleActionName}(${usedParamNames.join(', ')}${usedParamNames.length ? ', ' : ''}params);
            return JSONAPI.stringify(result, ${responseDtoName});
          } catch (err) {
            if (err instanceof JSONAPI.Exception) {
              throw JSONAPI.throwHttpException(err);
            } else if (!(err instanceof HttpException)) {
              throw new HttpException(String(err), 500);
            } else {
              throw err;
            }
          }
        `);
                break;
            case 'delete':
                actionFunc.addStatements(`
          try {
            const result = await this.${handleActionName}(${usedParamNames.join(', ')}${usedParamNames.length ? ', ' : ''});
            return JSONAPI.stringify(result, ${responseDtoName});
          } catch (err) {
            if (err instanceof JSONAPI.Exception) {
              throw JSONAPI.throwHttpException(err);
            } else if (!(err instanceof HttpException)) {
              throw new HttpException(String(err), 500);
            } else {
              throw err;
            }
          }
        `);
                break;
            default:
                assert_1.default.fail(`Unsupported method ${this.httpMethod}`);
        }
    }
    generateHandlerMethod(classDecl) {
        const handlerFunc = classDecl.addMethod({
            name: (0, api_tools_core_1.camelCase)(`handle-${this.methodName}`),
            returnType: `Promise<${(0, api_tools_core_1.titleCase)(`${this.methodName}Response`)}>`,
            isAsync: true,
            scope: ts_morph_1.Scope.Private,
        });
        for (const param of this.params) {
            handlerFunc.addParameter({
                name: param.name,
                type: 'string',
            });
        }
        handlerFunc.addStatements(`
      return 1;
    `);
    }
}
exports.Action = Action;
//# sourceMappingURL=Action.js.map