"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Controller = void 0;
const path_1 = __importDefault(require("path"));
const api_tools_core_1 = require("@fresha/api-tools-core");
const code_morph_ts_1 = require("@fresha/code-morph-ts");
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
const Action_1 = require("./Action");
/**
 * Generates code for a single NestJS controller.
 */
class Controller {
    /**
     * Given OpenAPI path item URL, returns controller class name.
     */
    static makeClassName(pathUrl) {
        const nameParts = (0, openapi_codegen_utils_1.significantNameParts)(pathUrl);
        return (0, api_tools_core_1.titleCase)(nameParts.concat('Controller').join('-')).replace(/\s+/g, '');
    }
    /**
     * Given an OpenAPI path item URL, returns controller file name.
     */
    static makeFileName(pathUrl) {
        const nameParts = (0, openapi_codegen_utils_1.significantNameParts)(pathUrl);
        if (nameParts.length) {
            return `${(0, openapi_codegen_utils_1.kebabCase)(nameParts.join('-'))}.controller.ts`;
        }
        return 'controller.ts';
    }
    context;
    className;
    outputPath;
    sourceFile;
    actions;
    urlPrefix;
    constructor(context, outputPath, className) {
        this.context = context;
        this.outputPath = outputPath;
        this.className = className;
        this.sourceFile = this.context.project.createSourceFile(this.outputPath, '', {
            overwrite: true,
        });
        this.actions = [];
        this.urlPrefix = null;
    }
    relativeModulePath(filePath) {
        return `./${path_1.default.relative(path_1.default.dirname(this.outputPath), path_1.default.join(path_1.default.dirname(filePath), path_1.default.basename(filePath, '.ts')))}`;
    }
    processPathItem(pathUrl, pathItem) {
        for (const [methodName, operation] of pathItem.operations()) {
            this.actions.push(new Action_1.Action(this.context, this, pathUrl, methodName, operation));
        }
        this.urlPrefix = null;
    }
    getUrlPrefix() {
        if (this.urlPrefix == null) {
            this.urlPrefix = (0, openapi_codegen_utils_1.commonStringPrefix)(this.actions.map(action => action.pathUrl));
        }
        return this.urlPrefix;
    }
    generateCode() {
        this.context.logger.info(`Generating controller code for ${this.outputPath}`);
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, '@nestjs/common', 'Controller');
        const classDecl = this.sourceFile.addClass({
            name: this.className,
            isExported: true,
        });
        (0, code_morph_ts_1.addDecorator)(classDecl, 'Controller', this.getUrlPrefix());
        for (const action of this.actions) {
            action.generateCode(classDecl);
        }
    }
}
exports.Controller = Controller;
//# sourceMappingURL=Controller.js.map