"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DTO = void 0;
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const api_tools_core_1 = require("@fresha/api-tools-core");
const code_morph_ts_1 = require("@fresha/code-morph-ts");
const openapi_codegen_utils_1 = require("@fresha/openapi-codegen-utils");
class DTO {
    context;
    className;
    outputPath;
    schema;
    sourceFile;
    constructor(context, name, schema) {
        this.context = context;
        this.className = name;
        this.outputPath = path_1.default.join(this.context.outputPath, 'dto', `${this.className}.dto.ts`);
        this.schema = schema;
        this.sourceFile = this.context.project.createSourceFile(this.outputPath, '', {
            overwrite: true,
        });
    }
    generateCode() {
        this.context.logger.info(`Generating DTO ${this.outputPath}`);
        this.addClassDecl(this.className, this.schema);
    }
    addClassDecl(className, schema) {
        const classDecl = this.sourceFile.addClass({
            name: className,
            isExported: true,
        });
        if (schema) {
            (0, assert_1.default)(schema.type === 'object', `Schema type '${String(schema.type)}'`);
            for (const prop of schema.getProperties()) {
                switch (prop.schema.type) {
                    case null:
                        this.addUnknownProperty(classDecl, prop);
                        break;
                    case 'boolean':
                        this.addBooleanProperty(classDecl, prop);
                        break;
                    case 'integer':
                    case 'number':
                        this.addNumericProperty(classDecl, prop);
                        break;
                    case 'string':
                        this.addStringProperty(classDecl, prop);
                        break;
                    case 'object':
                        this.addObjectProperty(classDecl, prop);
                        break;
                    case 'array':
                        this.addArrayProperty(classDecl, prop);
                        break;
                    default:
                        assert_1.default.fail(`Unexpected schema type: ${String(prop.schema.type)}`);
                }
            }
        }
    }
    addUnknownProperty(classDecl, prop) {
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: 'unknown',
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        if (prop.schema.allOfCount || prop.schema.oneOfCount || prop.schema.anyOfCount) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'ValidateNested');
            (0, code_morph_ts_1.addDecorator)(propDef, 'ValidateNested', undefined);
        }
    }
    addBooleanProperty(classDecl, prop) {
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: 'boolean',
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'IsBoolean');
        (0, code_morph_ts_1.addDecorator)(propDef, 'IsBoolean', undefined);
    }
    addNumericProperty(classDecl, prop) {
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: prop.schema.nullable ? 'number | null' : 'number',
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        const { min, max } = (0, openapi_codegen_utils_1.getNumericSchemaRange)(prop.schema);
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        if (min != null) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'Min');
            (0, code_morph_ts_1.addDecorator)(propDef, 'Min', min);
        }
        if (max != null) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'Max');
            (0, code_morph_ts_1.addDecorator)(propDef, 'Max', max);
        }
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'IsInt');
        (0, code_morph_ts_1.addDecorator)(propDef, 'IsInt', undefined);
    }
    addStringProperty(classDecl, prop) {
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: prop.schema.nullable ? 'string | null' : 'string',
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        if (prop.schema.minLength) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'MinLength');
            (0, code_morph_ts_1.addDecorator)(propDef, 'MinLength', prop.schema.minLength);
        }
        if (prop.schema.maxLength) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'MaxLength');
            (0, code_morph_ts_1.addDecorator)(propDef, 'MaxLength', prop.schema.maxLength);
        }
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'IsString');
        (0, code_morph_ts_1.addDecorator)(propDef, 'IsString', undefined);
    }
    addObjectProperty(classDecl, prop) {
        const propClassName = (0, api_tools_core_1.titleCase)(`${classDecl.getName() ?? ''}-${prop.name}`).replace(/\s+/g, '');
        this.addClassDecl(propClassName, prop.schema);
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: propClassName,
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Type');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        (0, code_morph_ts_1.addDecorator)(propDef, 'Type', (writer) => writer.write(`() => ${propClassName}`));
        (0, code_morph_ts_1.addDecorator)(propDef, 'ValidateNested', undefined);
    }
    addArrayProperty(classDecl, prop) {
        (0, assert_1.default)(this.schema);
        let itemTypeString = 'unknown[]';
        if (!Array.isArray(prop.schema.items)) {
            switch (prop.schema.items?.type) {
                case 'boolean':
                    itemTypeString = 'boolean[]';
                    break;
                case 'integer':
                case 'number':
                    itemTypeString = 'number[]';
                    break;
                case 'string':
                    itemTypeString = 'string[]';
                    break;
                case 'object':
                    itemTypeString = 'object[]';
                    break;
                case undefined:
                case null:
                    break;
                default:
                    assert_1.default.fail(`Unsupported schema type ${String(prop.schema.items?.type)}`);
            }
        }
        const propDef = classDecl.addProperty({
            name: prop.name,
            type: itemTypeString,
            hasQuestionToken: !prop.required,
        });
        propDef.prependWhitespace('\n');
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-transformer', 'Expose');
        (0, code_morph_ts_1.addDecorator)(propDef, 'Expose', undefined);
        if (prop.schema.minItems != null) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'ArrayMinSize');
            (0, code_morph_ts_1.addDecorator)(propDef, 'ArrayMinSize', prop.schema.minItems);
        }
        if (prop.schema.maxItems != null) {
            (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'ArrayMaxSize');
            (0, code_morph_ts_1.addDecorator)(propDef, 'ArrayMaxSize', prop.schema.maxItems);
        }
        (0, code_morph_ts_1.addImportDeclaration)(this.sourceFile, 'class-validator', 'IsArray');
        (0, code_morph_ts_1.addDecorator)(propDef, 'IsArray', undefined);
    }
}
exports.DTO = DTO;
//# sourceMappingURL=DTO.js.map